from time import sleep
from SX127x.LoRa import *
from SX127x.board_config import BOARD

print("\nThe very beginning...\n")
BOARD.setup()

class LoRaRcvCont(LoRa):
    def __init__(self, verbose=False):
        super(LoRaRcvCont, self).__init__(verbose)
        self.set_mode(MODE.SLEEP)
        self.set_dio_mapping([0] * 6)

    def start(self):
        self.reset_ptr_rx()
        self.set_mode(MODE.RXCONT)
        while True:
            sleep(.5)
            rssi_value = self.get_rssi_value()
            status = self.get_modem_status()
            sys.stdout.flush()
            
    def on_rx_done(self):
        print("\nReceived: ")
        self.clear_irq_flags(RxDone=1)
        payload = self.read_payload(nocheck=True)
        try:
            print(bytes(payload).hex())
        except (IndexError):
            print("Packet Index Error: Most likely a transmission collision")
        self.print_packet(payload)
        self.set_mode(MODE.SLEEP)
        self.reset_ptr_rx()
        self.set_mode(MODE.RXCONT) 

    def print_packet(self, payload):
        import struct
        
        print("RSSI: ", self.get_rssi_value(), "    Pkt RSSI: ", self.get_pkt_rssi_value(), "    Pkt SNR: ", self.get_pkt_snr_value())
        
        print("\nDestination MAC : ", end =" ")
        for i in range(0,4):
            print('{:02x}'.format(payload[i]), end =" ")
        destinationNode = int.from_bytes(payload[0:4], byteorder='little')
        if destinationNode == 0xa16ef174:
            print("\n                :  LoRa Gateway 1", end =" ")
        elif destinationNode == 0xdc6ef174:
            print("\n                :  LoRa Gateway 1 [DC]", end =" ")
        elif destinationNode == 0xdc6ef75c:
            print("\n                :  LoRa Gateway 2 [DC]", end =" ")
        elif destinationNode == 0xdc45f8f0:
            print("\n                :  LoRa V2 Gateway 1", end =" ")
        elif destinationNode == 0xdc460914:
            print("\n                :  LoRa V2 Gateway 2", end =" ")
        elif destinationNode == 0xdc80649c:
            print("\n                :  LoRa V3 Gateway 1", end =" ")
        elif destinationNode == 0xdc815740:
            print("\n                :  LoRa V3 Gateway 2", end =" ")
        elif destinationNode == 0xdc822940:
            print("\n                :  Stick 3", end =" ")
        elif destinationNode == 0xdc822d9c:
            print("\n                :  Stick 4", end =" ")
        elif destinationNode == 0xdc032f31:
            print("\n                :  CubeCell 1", end =" ")
        elif destinationNode == 0xdca7290f:
            print("\n                :  Pump Cell Plus", end =" ")
        else:
            print("\n                :  Unrecognised Destination", end =" ")
            
        print("\n     Source MAC : ", end =" ")
        for i in range(4,8):
            print('{:02x}'.format(payload[i]), end =" ")
        sourceNode = int.from_bytes(payload[4:8], byteorder='little')
        if sourceNode == 0xa16ef174:
            print("\n                :  LoRa Gateway", end =" ")
        elif sourceNode == 0xa16ef75c:
            print("\n                :  Node 32 Power", end =" ")
        elif sourceNode == 0xdc45f8f0:
            print("\n                :  LoRa V2 Gateway 1", end =" ")
        elif sourceNode == 0xdc460914:
            print("\n                :  LoRa V2 Gateway 2", end =" ")
        elif sourceNode == 0xdc80649c:
            print("\n                :  LoRa V3 Gateway 1", end =" ")
        elif sourceNode == 0xdc815740:
            print("\n                :  LoRa V3 Gateway 2", end =" ")
        elif sourceNode == 0xdc517110:
            print("\n                :  Rain Stick", end =" ")
        elif sourceNode == 0xdc51739c:
            print("\n                :  Tank Stick", end =" ")
        elif sourceNode == 0xdc822940:
            print("\n                :  Stick 3", end =" ")
        elif sourceNode == 0xdc822d9c:
            print("\n                :  Stick 4", end =" ")
        elif sourceNode == 0xdc125398:
            print("\n                :  Study ESP32 BME", end =" ")
        elif sourceNode == 0xdc125784:
            print("\n                :  Rover ESP32 BME", end =" ")
        elif sourceNode == 0xdc6a5728:
            print("\n                :  ESP32-WROOM-32-1", end =" ")
        elif sourceNode == 0xdc6a56ec:
            print("\n                :  ESP32-WROOM-32-2", end =" ")
        elif sourceNode == 0xdc48870c:
            print("\n                :  ESP32-WROOM-32-3 BME", end =" ")
        elif sourceNode == 0xdc488d6c:
            print("\n                :  ESP32-WROOM-32-4 BME", end =" ")
        elif sourceNode == 0xdc488710:
            print("\n                :  ESP32-WROOM-32-5", end =" ")
        elif sourceNode == 0xdc032f31:
            print("\n                :  CubeCell 1", end =" ")
        elif sourceNode == 0xdc531410:
            print("\n                :  CubeCell 2", end =" ")
        elif sourceNode == 0xdc632207:
            print("\n                :  CubeCell 3", end =" ")
        elif sourceNode == 0xdc772e37:
            print("\n                :  CubeCell 4", end =" ")
        elif sourceNode == 0xdca7290f:
            print("\n                :  Pump Cell Plus", end =" ")
        elif sourceNode == 0xdcb32836:
            print("\n                :  Weather Station", end =" ")
        elif sourceNode == 0xdcb32a0b:
            print("\n                :  AWTS", end =" ")
        elif sourceNode == 0xdc12051b:
            print("\n                :  BCS-A4a1 Mini", end =" ")
        elif sourceNode == 0xdce00002:
            print("\n                :  BCS-A4a2 Mini", end =" ")
        elif sourceNode == 0xdc11142c:
            print("\n                :  Porch Mini BME", end =" ")
        elif sourceNode == 0xdc112316:
            print("\n                :  Shed Mini BME", end =" ")
        elif sourceNode == 0xdc112b0d:
            print("\n                :  BCS-A4a5 Mini", end =" ")
        elif sourceNode == 0xdc12111d:
            print("\n                :  BCS-A4a6 Mini", end =" ")
        elif sourceNode == 0xdc121123:
            print("\n                :  BCS-A4a7 Mini", end =" ")
        elif sourceNode == 0xdc120619:
            print("\n                :  BCS-A4a8 Mini", end =" ")
        elif sourceNode == 0xdca541e3:
            print("\n                :  NodeMCU-1", end =" ")
        elif sourceNode == 0xdc120f38:
            print("\n                :  NodeMCU-3", end =" ")
        elif sourceNode == 0xdc8dbaac:
            print("\n                :  NodeMCU-5", end =" ")
        elif sourceNode == 0xdc296d39:
            print("\n                :  ESP-12S BME", end =" ")
        elif sourceNode == 0xdcb600ea:
            print("\n                :  ESP-12F-03000001", end =" ")
        elif sourceNode == 0xdcb61052:
            print("\n                :  ESP-12F-03000002", end =" ")
        elif sourceNode == 0xdc2edf35:
            print("\n                :  ESP-12F-03000003", end =" ")
        elif sourceNode == 0xdc2ee2f7:
            print("\n                :  ESP-12F-03000004", end =" ")
        elif sourceNode == 0xdc29d4de:
            print("\n                :  ESP-12F-03010001", end =" ")
        elif sourceNode == 0xdc2b5d56:
            print("\n                :  ESP-12F-030xxxxx", end =" ")
        else:
            print("\n                :  Unrecognised Source", end =" ")
            
        print("\n            CRC : ", end =" ")
        for i in range(8,12):
            print('{:02x}'.format(payload[i]), end =" ")
            
        print("\n     Sequence # : ", end =" ")
        for i in range(12,14):
            print('{:02x}'.format(payload[i]), end =" ")
        print("\n                : ", int.from_bytes(payload[12:14], byteorder='little'), end =" ")
        
        packetType = payload[14]
        payloadLength = payload[15]
        relMask = 0x80
        byteMask = 0x7F
        if packetType & relMask:
            payload[14] = payload[14] & byteMask
            if payloadLength & relMask:
                print("\n       Reliable :  ACK", end =" ")
                payload[15] = payload[15] & byteMask
            else:
                print("\n       Reliable :  REL", end =" ")
            
        print("\n           Type : ", '{:02x}'.format(payload[14]))        
        validType = True
        if payload[14] == 0x01:
            print("                :  Voltage", end =" ")
        elif payload[14] == 0x02:
            print("                :  Power", end =" ")
        elif payload[14] == 0x11:
            print("                :  Tank", end =" ")
        elif payload[14] == 0x12:
            print("                :  Pump", end =" ")
        elif payload[14] == 0x20:
            print("                :  Weather", end =" ")
        elif payload[14] == 0x21:
            print("                :  Atmosphere", end =" ")
        elif payload[14] == 0x27:
            print("                :  Temperature", end =" ")
        elif payload[14] == 0x22:
            print("                :  Rainfall", end =" ")
        elif payload[14] == 0x23:
            print("                :  Wind", end =" ")
        elif payload[14] == 0x24:
            print("                :  VOX", end =" ")
        elif payload[14] == 0x25:
            print("                :  Light", end =" ")
        elif payload[14] == 0x26:
            print("                :  UV", end =" ")
        elif payload[14] == 0x30:
            print("                :  Sprinkler", end =" ")
        elif payload[14] == 0x40:
            print("                :  GPS", end =" ")
        elif payload[14] == 0x50:
            print("                :  AWTS", end =" ")
        elif payload[14] == 0xF0:
            print("                :  Reset", end =" ")
        else:
            print("                :  Unrecognised", end =" ")
            validType = False
            
        if payload[15] == 0:
            validLength = False
        else:
            validLength = True        
            
        if validType and validLength:    
            print("\n Payload Length : ", '{:02x}'.format(payload[15]))
            print("                : ", int.from_bytes(payload[15:16], byteorder='little'))
            print("        Payload : ", end =" ")
            for i in range(16,(16+int.from_bytes(payload[15:16], byteorder='little'))):
                print('{:02x}'.format(payload[i]), end =" ")
            print("")
            if payload[14] == 0x01:
#                print("                : ","Voltage")
                print("        Voltage : ", "{:.2f}".format(int.from_bytes(payload[16:18], byteorder='little')/1000), "V", end =" ")
            elif payload[14] == 0x02:
#                print("                : ","Power")
                print("Battery Voltage : ", "{:.2f}".format(int.from_bytes(payload[16:18], byteorder='little')/1000), "V")
                print("Battery Current : ", int.from_bytes(payload[18:20], byteorder='little'), "mA")
                print("  Panel Voltage : ", "{:.2f}".format(int.from_bytes(payload[20:22], byteorder='little')/1000), "V")
                print("  Panel Current : ", int.from_bytes(payload[22:24], byteorder='little'), "mA")
                print("   Load Voltage : ", "{:.2f}".format(int.from_bytes(payload[24:26], byteorder='little')/1000), "V")
                print("   Load Current : ", int.from_bytes(payload[26:28], byteorder='little'), "mA", end =" ")
            elif payload[14] == 0x11:
#                print("                : ","Tank")
                print("             ID : ", int.from_bytes(payload[16:17], byteorder='little'))
                print("    Level Index : ", int.from_bytes(payload[17:19], byteorder='little'), end =" ")
            elif payload[14] == 0x12:
#                print("                : ","Pump")
                print("             ID : ", int.from_bytes(payload[16:17], byteorder='little'))
                print("          Power : ", int.from_bytes(payload[17:18], byteorder='little'), end =" ")
            elif payload[14] == 0x20:
#                print("                : ","Weather")
                print("    Temperature : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little', signed='True')/10), "°C")
                print("       Pressure : ", int.from_bytes(payload[18:20], byteorder='little'), "hPa")
                print("       Humidity : ", int.from_bytes(payload[20:22], byteorder='little'), "%")
                print("       Rainfall : ", "{:.1f}".format(int.from_bytes(payload[22:24], byteorder='little')/10), "mm")
                print(" Wind Direction : ", int.from_bytes(payload[24:26], byteorder='little'), "°")
                print("     Wind Speed : ", int.from_bytes(payload[26:28], byteorder='little'), "kph", end =" ")
            elif payload[14] == 0x21:
#                print("                : ","Atmosphere")
                print("    Temperature : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little', signed='True')/10), "°C")
                print("       Pressure : ", int.from_bytes(payload[18:20], byteorder='little'), "hPa")
                print("       Humidity : ", int.from_bytes(payload[20:22], byteorder='little'), "%", end =" ")
            elif payload[14] == 0x27:
#                print("                : ","Temperature")
                print("    Temperature : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little', signed='True')/10), "°C", end =" ")
            elif payload[14] == 0x22:
#                print("                : ","Rainfall")
                print("       Rainfall : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little')/5), "mm", end =" ")
            elif payload[14] == 0x23:
#                print("                : ","Wind")
                print("      Direction : ", int.from_bytes(payload[16:18], byteorder='little'), "°")
                print("          Speed : ", int.from_bytes(payload[18:19], byteorder='little'), "kph", end =" ")
            elif payload[14] == 0x24:
#                print("                : ","VOX")
                print("            VOX : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little')/10), "ppm", end =" ")
            elif payload[14] == 0x25:
#                print("                : ","Light")
                print("        Ambient : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little')/100), "lux", end =" ")
            elif payload[14] == 0x26:
#                print("                : ","UV")
                print("            UVA : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little')/100))
                print("            UVB : ", "{:.1f}".format(int.from_bytes(payload[18:20], byteorder='little')/100))
                print("       UV Index : ", "{:.1f}".format(int.from_bytes(payload[20:22], byteorder='little')/100), end =" ")
            elif payload[14] == 0x40:
#                print("                : ","GPS")
                print("       Latitude : ", "{:.1f}".format(int.from_bytes(payload[16:18], byteorder='little')/10), "°")
                print("      Longitude : ", "{:.1f}".format(int.from_bytes(payload[18:20], byteorder='little')/10), "°")
                print("       Altitude : ", "{:.1f}".format(int.from_bytes(payload[20:22], byteorder='little')/10), "m")
                print("           HDOP : ", "{:.1f}".format(int.from_bytes(payload[22:24], byteorder='little')/10), "", end =" ")
            elif payload[14] == 0x50:
#                print("                : ","AWTS")
                print("       Pressure : ", "{:.2f}".format(int.from_bytes(payload[16:18], byteorder='little')/100), "kPa")
                print("          Level : ", int.from_bytes(payload[18:19], byteorder='little'), "%", end =" ")
            elif payload[14] == 0xF0:
#                print("                : ","Reset")
                print("     Error Code : ", int.from_bytes(payload[16:17], byteorder='little'), "", end =" ")
                
        print("\n")

lora = LoRaRcvCont(verbose=True)
lora.set_mode(MODE.STDBY)

#  Medium Range  Defaults after init should be 917.0MHz, Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on 13 dBm

lora.set_pa_config(pa_select=1)
lora.set_freq(917)
print("Set Frequency: ",lora.get_freq(), "MHz")
lora.set_sync_word(0x12)
print("Set Sync Word: ",hex(lora.get_sync_word()))

try:
    lora.start()
except KeyboardInterrupt:
    sys.stdout.flush()
    print("")
    sys.stderr.write("KeyboardInterrupt\n")
finally:
    sys.stdout.flush()
    print("")
    lora.set_mode(MODE.SLEEP)
    BOARD.teardown()
